# -*- coding: UTF-8 -*-
# Date    : 2018/08/07
# Author  : Snow Yang
# Mail    : yangsw@mxchip.com

import os
import sys
import shlex
import struct
import argparse
from subprocess import Popen, CalledProcessError, PIPE
from .progressbar import ProgressBar

MARKER = {'osx': '█', 'win': '#'}
FILL = {'osx': '░', 'win': '-'}

PREFIX = '-->'
prefix_len = len(PREFIX)

def flasher(curdir, mcu, file_name, addr, debugger):

    hostos = 'osx' if sys.platform == 'darwin' else 'Linux64' if sys.platform == 'linux2' else 'win'
    pbar = ProgressBar(os.path.getsize(file_name), marker=MARKER[hostos], fill=FILL[hostos])
    openocd = os.path.join(curdir, 'openocd', hostos, 'openocd_mxos')

    cmd_line = openocd + \
        ' -s ' + curdir + \
        ' -f ' + os.path.join(curdir, 'interface', debugger + '.cfg') + \
        ' -f ' + os.path.join(curdir, 'targets', mcu + '.cfg') + \
        ' -f ' + os.path.join(curdir, 'flashloader', 'scripts', 'flash.tcl') + \
        ' -f ' + os.path.join(curdir, 'flashloader', 'scripts', 'cmd.tcl') + \
        ' -c init' + \
        ' -c flash_alg_pre_init' + \
        ' -c "flash_alg_init ' + os.path.join(curdir, 'flashloader', 'ramcode', mcu + '.elf').replace('\\', '/') + '"' + \
        ' -c "write ' + file_name.replace('\\', '/') + ' ' + addr + '" -c shutdown'
    proc = Popen(cmd_line, shell=True, universal_newlines=True, stderr=PIPE)
    logtext = ''

    while True:
        out = proc.stderr.readline().strip()
        logtext += out + '\r\n'
        if proc.poll() != None:
            if proc.poll():
                print(logtext)
            return
        else:
            if out[:prefix_len] == PREFIX:
                pos = int(out[prefix_len:], 0)
                pbar.update(pos)

long_description = '''MXCHIP Flash Tool.

Author  : Snow Yang
Mail    : yangsw@mxchip.com
Version : 1.2.5
'''

def main():
    parser = argparse.ArgumentParser(description=long_description, formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('-m', '--mcu', type=str, required=True, help='mcu name')
    parser.add_argument('-f', '--file', type=str, required=True, help='file name')
    parser.add_argument('-a', '--addr', type=str, required=True, help='address')
    parser.add_argument('-d', '--debugger', type=str, required=False, default='jlink_swd', help='address')

    curdir = os.path.join(os.path.dirname(os.path.abspath(__file__)))
    args = parser.parse_args(sys.argv[1:])

    flasher(curdir, args.mcu, args.file, args.addr, args.debugger)

def interactive():
    print(long_description)

    if len(sys.argv) == 1:
        print('mflashi <file name>')
        return 1
    curdir = os.path.join(os.path.dirname(os.path.abspath(__file__)))
    mcu_list = []
    for root, dirs, files in os.walk(os.path.join(curdir, 'targets')):
        for name in files:
            if name.endswith('.cfg'):
                mcu_list.append(os.path.splitext(name)[0])
    _file = sys.argv[1]
    print('MCU list:')
    for i, name in enumerate(mcu_list):
        print(' %d - %s'%(i, name))
    mcu = mcu_list[int(input('MCU Number: '))]
    addr = input('Download Address: ')

    flasher(curdir, mcu, _file, addr)

    input('Press any key to exit: ')

if __name__ == "__main__":
    main()


# -*- coding: UTF-8 -*-
# Date    : 2018/08/07
# Author  : Snow Yang
# Mail    : yangsw@mxchip.com

import sys
from time import sleep, perf_counter
from .terminal import get_terminal_size


class ProgressBar():
    def __init__(self, max_value, marker='█', fill='░'):
        self.marker = marker
        self.max_value = max_value
        self.fill = fill
        w = get_terminal_size()[0]
        self.width = w - 5 - 9 - 12 - 9
        self.last_width = 0
        self.last_value = 0
        self.last_time = 0
        self.start_time = 0

    def update(self, value):
        if value > self.max_value:
            raise ValueError('value to large')
        marker_width = value * self.width // self.max_value
        if marker_width <= self.last_width:
            return
        speed = 0
        t = perf_counter()
        if self.last_time > 0:
            speed = (value - self.last_value) / (t - self.last_time)
        else:
            self.start_time = t
        self.last_value = value
        self.last_time = t
        self.last_width = marker_width
        fill_width = self.width - marker_width
        sys.stdout.write('\r' + self.marker * marker_width +
                         self.fill * fill_width + ' %3d' % (value * 100 / self.max_value) + '%' +
                         ' %4d KiB' % (value / 1024) +
                         ' %5.1f KiB/s' % (speed / 1024) +
                         ' %4d s' % (t - self.start_time))
        if value == self.max_value:
            sys.stdout.write('\r\n')
        sys.stdout.flush()

if __name__ == "__main__":
    bar = ProgressBar(1024 * 36)
    for i in range(36):
        bar.update((i + 1) * 1024)
        sleep(0.1)



# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path, system
# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
from io import open
import subprocess
import sys
import webbrowser

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    # This is the name of your project. The first time you publish this
    # package, this name will be registered for you. It will determine how
    # users can install this project, e.g.:
    #
    # $ pip install sampleproject
    #
    # And where it will live on PyPI: https://pypi.org/project/sampleproject/
    #
    # There are some restrictions on what makes a valid project name
    # specification here:
    # https://packaging.python.org/specifications/core-metadata/#name
    name='mflash',  # Required

    # Versions should comply with PEP 440:
    # https://www.python.org/dev/peps/pep-0440/
    #
    # For a discussion on single-sourcing the version across setup.py and the
    # project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='1.2.5',  # Required

    # This is a one-line description or tagline of what your project does. This
    # corresponds to the "Summary" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#summary
    description='MXCHIP Flash Tool',  # Optional

    # This is an optional longer description of your project that represents
    # the body of text which users will see when they visit PyPI.
    #
    # Often, this is the same as your README, so you can just read it in from
    # that file directly (as we have already done above)
    #
    # This field corresponds to the "Description" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#description-optional
    long_description=long_description,  # Optional

    # Denotes that our long_description is in Markdown; valid values are
    # text/plain, text/x-rst, and text/markdown
    #
    # Optional if long_description is written in reStructuredText (rst) but
    # required for plain-text or Markdown; if unspecified, "applications should
    # attempt to render [the long_description] as text/x-rst; charset=UTF-8 and
    # fall back to text/plain if it is not valid rst" (see link below)
    #
    # This field corresponds to the "Description-Content-Type" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#description-content-type-optional
    long_description_content_type='text/markdown',  # Optional (see note above)

    # This should be a valid link to your project's main homepage.
    #
    # This field corresponds to the "Home-Page" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#home-page-optional
    url='https://github.com/smartsnow/mflash',  # Optional

    # This should be your name or the name of the organization which owns the
    # project.
    author='Snow Yang',  # Optional

    # This should be a valid email address corresponding to the author listed
    # above.
    author_email='yangsw@mxchip.com',  # Optional

    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        # These classifiers are *not* checked by 'pip install'. See instead
        # 'python_requires' below.
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],

    # This field adds keywords for your project which will appear on the
    # project page. What does your project relate to?
    #
    # Note that this is a string of words separated by whitespace, not a list.
    keywords='MXCHIP Flash Tool',  # Optional

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(),  # Required

    # Specify which Python versions you support. In contrast to the
    # 'Programming Language' classifiers above, 'pip install' will check this
    # and refuse to install the project if the version does not match. If you
    # do not support Python 2, you can simplify this to '>=3.5' or similar, see
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/#python-requires
    python_requires='>=3.5',

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['qdarkstyle==2.7', 'pyqt5==5.10.1'],  # Optional

    # List additional groups of dependencies here (e.g. development
    # dependencies). Users will be able to install these using the "extras"
    # syntax, for example:
    #
    #   $ pip install sampleproject[dev]
    #
    # Similar to `install_requires` above, these must be valid existing
    # projects.
    extras_require={  # Optional
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.
    #
    # If using Python 2.6 or earlier, then these have to be included in
    # MANIFEST.in as well.
    package_data={  # Optional
        'mflash': ['interface/swj-dp.tcl',
            'interface/stlink-v2.cfg',
            'interface/stlink-v2-1.cfg',
            'interface/jlink_swd.cfg',
            'openocd/osx/libusb-0.1.4.dylib',
            'openocd/osx/openocd_mxos_run',
            'openocd/osx/openocd_mxos_dirname',
            'openocd/osx/openocd_mxos',
            'openocd/osx/libusb-1.0.0.dylib',
            'openocd/osx/libftdi1.2.dylib',
            'openocd/osx/libhidapi.0.dylib',
            'openocd/win/libusb-0-1-4.dll',
            'openocd/win/libusb-1.0.dll',
            'openocd/win/libhidapi-0.dll',
            'openocd/win/openocd_mxos.exe',
            'openocd/win/libftdi1.dll',
            'targets/mx1270.cfg',
            'targets/mx1290.cfg',
            'targets/mx1410.cfg',
            'flashloader/ramcode/mx1290.elf',
            'flashloader/ramcode/mx1270.elf',
            'flashloader/ramcode/mx1410.elf',
            'flashloader/scripts/cmd.tcl',
            'flashloader/scripts/flash.tcl',
            'resources/flash.png',
            'resources/download.png',
            'resources/log.png',
            'resources/help.png',
            'resources/connection.png',
            'resources/jlink_swd.png',
            'resources/stlink-v2-1.png',
            'resources/stlink-v2.png',
            'resources/open.png',
        ]
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[
    ],  # Optional

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    entry_points={  # Optional
        'console_scripts': [
            'mflash=mflash.main:main',
            'mflashi=mflash.ui:main',
        ],
    },

    scripts = ['start.vbs']
)

if sys.platform == 'win32':
    # if not register
    curdir = path.join(sys.prefix, 'Scripts')
    exefile = path.join(curdir, 'start.vbs')
    regfile = path.join(curdir, 'rightclick.reg')
    regdata = 'Windows Registry Editor Version 5.00\r\n' + \
        '[HKEY_CLASSES_ROOT\*\shell\mflash]\r\n' + \
        '@="mflash - Download"\r\n' + \
        '[HKEY_CLASSES_ROOT\*\shell\mflash\command]\r\n' + \
        '@=hex(2):' + ',00,'.join([i.to_bytes(1, 'big').hex() for i in ('Wscript.exe "'+exefile+'"' + ' "%1"').encode()]) + ',00'
    open(regfile, 'w').write(regdata)
    system(regfile)

webbrowser.open('https://www.mxchip.com')



import os
import sys
import qdarkstyle
from PyQt5.QtWidgets import *
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from subprocess import Popen, CalledProcessError, PIPE

class UserTableWidget(QTableWidget):

    def __init__(self):
        super().__init__()
        self.setColumnCount(2)
        self.verticalHeader().setVisible(False)
        self.horizontalHeader().setVisible(False)
        self.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.horizontalHeader().setStretchLastSection(True)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

    def addRow(self, *rowItems):
        rowPosition = self.rowCount()
        self.insertRow(rowPosition)
        for i in range(self.columnCount()):
            self.setItem(rowPosition, i, QTableWidgetItem(rowItems[i]))
        self.resizeRowToContents(rowPosition)

class HelpWidet(QWidget):

    def __init__(self):
        super().__init__()
        self.setWindowTitle('Help')
        vLayout = QVBoxLayout(self)
        vLayout.addWidget(QLabel('Information'))
        self.infoTable = UserTableWidget()
        vLayout.addWidget(self.infoTable)
        self.infoTable.addRow('Author', 'Snow Yang')
        self.infoTable.addRow('Mail', 'yangsw@mxchip.com')
        self.infoTable.addRow('Version', '1.2.5')
        self.infoTable.setMaximumHeight(self.infoTable.rowHeight(0) * 3.2)
        self.label = QLabel('')
        vLayout.addWidget(self.label)
        self.label.setScaledContents(True)
        curdir = os.path.join(os.path.dirname(os.path.abspath(__file__)))
        self.label.setPixmap(QPixmap(os.path.join(curdir, 'resources', 'connection.png')))
        vLayout.addStretch()

class UserWidget(QWidget):

    def __init__(self):
        super().__init__()
        # self.resize(200, 200)
        self.setWindowTitle('MXCHIP Flash Tool')
        curdir = os.path.join(os.path.dirname(os.path.abspath(__file__)))

        hLayout = QHBoxLayout(self)
        vLayout = QVBoxLayout()
        hLayout.addLayout(vLayout)
        h0Layout = QHBoxLayout()
        vLayout.addLayout(h0Layout)
        self.fileNameLineEdit = QLineEdit()
        h0Layout.addWidget(self.fileNameLineEdit)
        self.fileNameLineEdit.setReadOnly(True)
        self.openFileButton = QPushButton(QIcon(os.path.join(curdir, 'resources/open.png')), '')
        h0Layout.addWidget(self.openFileButton)
        self.openFileButton.setToolTip('Open File')
        h1Layout = QHBoxLayout()
        vLayout.addLayout(h1Layout)
        self.moduleComboBox = QComboBox()
        h1Layout.addWidget(self.moduleComboBox)
        self.debuggerComboBox = QComboBox()
        h1Layout.addWidget(self.debuggerComboBox)
        self.lineEdit = QLineEdit()
        h1Layout.addWidget(self.lineEdit)
        self.lineEdit.setText('0x00')
        self.lineEdit.setMinimumWidth(80)
        self.button = QPushButton(QIcon(os.path.join(curdir, 'resources/download.png')), '')
        self.button.setToolTip('Download')
        h1Layout.addWidget(self.button)
        h2Layout = QHBoxLayout()
        vLayout.addLayout(h2Layout)
        self.progressBar = QProgressBar()
        h2Layout.addWidget(self.progressBar)
        self.showHideLogButton = QPushButton(QIcon(os.path.join(curdir, 'resources/log.png')), '')
        h2Layout.addWidget(self.showHideLogButton)
        self.showHideLogButton.setToolTip('Show/Hide log')
        self.helpButton = QPushButton(QIcon(os.path.join(curdir, 'resources/help.png')), '')
        h2Layout.addWidget(self.helpButton)
        self.helpButton.setToolTip('Help')
        self.plainTextEdit = QPlainTextEdit()
        self.plainTextEdit.hide()
        vLayout.addWidget(self.plainTextEdit)
        v0Layout = QVBoxLayout()
        hLayout.addLayout(v0Layout)
        self.plainTextEdit.setReadOnly(True)
        self.picLabel = QLabel('')
        v0Layout.addWidget(self.picLabel)
        self.picLabel.setScaledContents(True)
        v0Layout.addStretch()

PREFIX = '-->'
prefix_len = len(PREFIX)

errorLogReasonDict = {
"in procedure 'ocd_bouncer'": 'Please check JTAG/SWD connection.',
'LIBUSB_ERROR_NOT_FOUND.': 'Please download "Zadig", and replace J-Link driver with "WinUSB".',
'Error: No J-Link device found.':'Please check J-Link USB connection.'
}

class Worker(QThread):

    signalSetProgressBarMax = pyqtSignal(int)
    signalProgressBar = pyqtSignal(int)
    signalPlainTextEdit = pyqtSignal(str)
    signalMessageBox = pyqtSignal(str)

    def __init__(self, widget):
        super().__init__()
        self.curdir = os.path.join(os.path.dirname(os.path.abspath(__file__)))

        self.widget = widget

        self.mculist = []
        for _root, _dirs, files in os.walk(os.path.join(self.curdir, 'targets')):
            for name in files:
                if name.endswith('.cfg'):
                    self.mculist.append(os.path.splitext(name)[0])
        self.widget.moduleComboBox.addItems(self.mculist)

        self.deubbgerList = []
        for _root, _dirs, files in os.walk(os.path.join(self.curdir, 'interface')):
            for name in files:
                if name.endswith('.cfg'):
                    self.deubbgerList.append(os.path.splitext(name)[0])
        self.widget.debuggerComboBox.addItems(self.deubbgerList)

        hostos = 'osx' if sys.platform == 'darwin' else 'Linux64' if sys.platform == 'linux2' else 'win'
        self.openocd = os.path.join(self.curdir, 'openocd', hostos, 'openocd_mxos')

        self.semaphore = QSemaphore()
        self.widget.button.clicked.connect(self.download)
        self.logShow = False
        self.widget.showHideLogButton.clicked.connect(self.showHideLog)
        self.widget.helpButton.clicked.connect(self.showHelp)
        self.widget.openFileButton.clicked.connect(self.openFile)
        self.widget.debuggerComboBox.currentTextChanged.connect(self.showDebugger)

        self.signalSetProgressBarMax.connect(lambda value: self.widget.progressBar.setMaximum(value))
        self.signalProgressBar.connect(lambda value: self.widget.progressBar.setValue(value))
        self.signalPlainTextEdit.connect(lambda text: self.widget.plainTextEdit.appendPlainText(text))
        self.signalMessageBox.connect(lambda text: QMessageBox.critical(self.widget, '', text, QMessageBox.Yes, QMessageBox.Yes))

        self.widget.progressBar.setMinimum(0)

        self.widget.debuggerComboBox.setCurrentText('jlink_swd')
        self.showDebugger('jlink_swd')

        self.helpWidet = HelpWidet()

    def openFile(self):
        filename, filetype = QFileDialog.getOpenFileName(self.widget, "Open Image", os.path.dirname(self.widget.fileNameLineEdit.text()), "Binary file (*.bin)")
        if not filename:
            return
        self.widget.fileNameLineEdit.setText(filename)

    def showDebugger(self, text):
        self.widget.picLabel.setPixmap(QPixmap(os.path.join(self.curdir, 'resources', text + '.png')))

    def showHelp(self):
        self.helpWidet.show()

    def showHideLog(self):
        if self.logShow:
            self.widget.plainTextEdit.hide()
            self.logShow = False
        else:
            self.widget.plainTextEdit.show()
            self.logShow = True
        self.widget.adjustSize()

    def download(self):
        self.widget.moduleComboBox.setEnabled(False)
        self.widget.debuggerComboBox.setEnabled(False)
        self.widget.lineEdit.setEnabled(False)
        self.widget.button.setEnabled(False)
        self.widget.openFileButton.setEnabled(False)
        self.semaphore.release()

    def run(self):
        while True:
            self.semaphore.acquire()
            mcu = self.widget.moduleComboBox.currentText()
            debugger = self.widget.debuggerComboBox.currentText()
            addr = self.widget.lineEdit.text()
            filename = self.widget.fileNameLineEdit.text()
            self.signalSetProgressBarMax.emit(os.path.getsize(filename))

            cmd_line = self.openocd + \
                ' -s ' + self.curdir + \
                ' -f ' + os.path.join(self.curdir, 'interface', debugger + '.cfg') + \
                ' -f ' + os.path.join(self.curdir, 'targets', mcu + '.cfg') + \
                ' -f ' + os.path.join(self.curdir, 'flashloader', 'scripts', 'flash.tcl') + \
                ' -f ' + os.path.join(self.curdir, 'flashloader', 'scripts', 'cmd.tcl') + \
                ' -c init' + \
                ' -c flash_alg_pre_init' + \
                ' -c "flash_alg_init ' + os.path.join(self.curdir, 'flashloader', 'ramcode', mcu + '.elf').replace('\\', '/') + '"' + \
                ' -c "write ' + \
                filename.replace('\\', '/') + ' ' + addr + '" -c shutdown'
            proc = Popen(cmd_line, shell=True, universal_newlines=True, stderr=PIPE)

            log = ''
            while True:
                out = proc.stderr.readline().strip()
                log += out
                self.signalPlainTextEdit.emit(out)
                if proc.poll() != None:
                    break
                else:
                    if out[:prefix_len] == PREFIX:
                        self.signalProgressBar.emit(int(out[prefix_len:], 0))

            if proc.poll():
                reason = 'Unkown Reason'
                for errorLog in errorLogReasonDict:
                    if errorLog in log:
                        reason = errorLogReasonDict[errorLog]
                        break
                self.signalMessageBox.emit('Download Failed!\r\n\r\nReason:\r\n%s' % (reason))
            self.widget.moduleComboBox.setEnabled(True)
            self.widget.debuggerComboBox.setEnabled(True)
            self.widget.lineEdit.setEnabled(True)
            self.widget.button.setEnabled(True)
            self.widget.openFileButton.setEnabled(True)

def main():
    app = QApplication(sys.argv)
    app.setFont(QFont("Calibri"))
    app.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
    curdir = os.path.join(os.path.dirname(os.path.abspath(__file__)))
    app.setWindowIcon(QIcon(os.path.join(curdir, 'resources/flash.png')))
    widget = UserWidget()
    widget.fileNameLineEdit.setText(sys.argv[1])
    widget.show()
    worker = Worker(widget)
    worker.start()
    sys.exit(app.exec_())

if __name__ == "__main__":
    main()
